export interface InstalledHook {
    fn: Function;
    delegateTo: InstalledHook | undefined;
}
/**
 * Hooking a function means replacing it with our own implementation that (optionally)
 * delegates to the original.
 *
 * Multiple hooks should be allowed, delegating in a chain.
 *
 * If we install hook A, then install hook B, then uninstall A, hopefully this should not break.
 * However it requires that hook B appropriately skips over the now-uninstalled A.
 *
 * This manager wraps up that behavior.
 */
declare abstract class HookManager {
    constructor();
    /** Is our handler installed? */
    private _installed;
    /** backup of original function being hooked */
    private _original;
    /** our replacement, handles hook invocation */
    private _handler;
    private _hooks;
    abstract getFunction(): Function;
    abstract setFunction(fn: Function): void;
    /**
     * Install a hook.
     * Returns a function the hook can call to delegate to the next in the chain,
     * like a `super.foo()` call.
     */
    installHook(hookFn: Function): Function;
    /**
     * Uninstall a previously installed hook.
     */
    uninstallHook(hookFn: Function): void;
}
declare class ResolveFilenameHookManager extends HookManager {
    getFunction(): any;
    setFunction(fn: Function): void;
}
export declare const _resolveFilenameHookManager: ResolveFilenameHookManager;
export {};
//# sourceMappingURL=core.d.ts.map